
#include "windows.h"
#include "stdio.h"
#include "architecture/PainterEngine.h"
#include "platform/modules/px_time.h"

extern int PX_AudioInitializeHwnd(HWND hwnd);
extern int PX_AudioInitialize(PX_SoundPlay* soundPlay);
PX_SoundPlay soundPlay;
px_char runtime_buffer[1024 * 1024 * 8] = {0};
px_char script[1024 * 1024] = {0};
PX_PianoModel pianoModel;
PX_Compiler compiler;
PX_VM vmInstance;

px_void PX_PianoRead(px_void* userptr, px_byte* pBuffer, px_int readSize)
{
	if (!readSize || readSize > 2048 * 2 * 2)
	{
		PX_memset(pBuffer, 0, readSize);
	}
	else
	{
		px_int oft = 0;
		px_int count = readSize / 4;//2bytes 2channels
		px_float out44100f[2048];
		px_int i;
		px_short* PCM = (px_short*)pBuffer;
		PX_PianoModelGo(&pianoModel, out44100f, count);
		for (i = 0; i < count; i++)
		{
			PCM[oft++] = (px_short)(out44100f[i] * 32768 * 8);
			PCM[oft++] = (px_short)(out44100f[i] * 32768 * 8);
		}
	}
}



px_bool PX_VM_PianoKeyDown(PX_VM* Ins, px_void* ptr)
{
	if (PX_VM_STACK(Ins, 0).type != PX_VM_VARIABLE_TYPE_STRING)
	{
		PX_VM_RET(Ins, PX_VM_Variable_int(0));
		return PX_TRUE;
	}
	PX_PianoModelTriggerKey(&pianoModel, PX_VM_STACK(Ins, 0)._string.buffer);
	printf(PX_VM_STACK(Ins, 0)._string.buffer);
	printf("\n");
	return PX_TRUE;
}
px_bool PX_VM_PianoKeyDownIndex(PX_VM* Ins, px_void* ptr)
{
	if (PX_VM_STACK(Ins, 0).type != PX_VM_VARIABLE_TYPE_INT)
	{
		PX_VM_RET(Ins, PX_VM_Variable_int(0));
		return PX_TRUE;
	}
	PX_PianoModelTriggerIndex(&pianoModel, PX_VM_STACK(Ins, 0)._int);
	
	return PX_TRUE;
}

px_bool PX_VM_Wait(PX_VM* Ins, px_void* ptr)
{
	if (PX_VM_STACK(Ins, 0).type != PX_VM_VARIABLE_TYPE_INT)
	{
		PX_VM_RET(Ins, PX_VM_Variable_int(0));
		return PX_TRUE;
	}
	Ins->pThread[Ins->T].sleep = PX_VM_STACK(Ins, 0)._int;
	PX_VM_RET(Ins, PX_VM_Variable_int(0));
	return PX_TRUE;
}

px_bool PX_VM_CreateThread(PX_VM* Ins, px_void* userptr)
{

	if (PX_VM_STACK(Ins, 0).type != PX_VM_VARIABLE_TYPE_STRING)
	{
		PX_VM_RET(Ins, PX_VM_Variable_int(0));
		return PX_TRUE;
	}

	if (!PX_VMBeginThreadFunction(&vmInstance, PX_VMGetFreeThreadId(Ins), PX_VM_STACK(Ins, 0)._string.buffer, PX_NULL, 0))
	{
		PX_VM_RET(Ins, PX_VM_Variable_int(0));
	}
	return PX_TRUE;
}


int main()
{
	FILE* pf;
	px_memory payload;
	px_memorypool mp = MP_Create(runtime_buffer, sizeof(runtime_buffer));
	px_dword now, lastUpdateTime;
	printf("PainterEngine Piano Executer\n");
	printf("load player script\n");
	fopen_s(&pf,"piano.txt", "rb");
	if (!pf)
	{
		printf("player script not found\n");
		return PX_FALSE;
	}
	fread(script, 1, sizeof(script), pf);
	fclose(pf);
	printf("compiling....\n");
	PX_MemoryInitialize(&mp, &payload);
	if (!PX_CompilerInitialize(&mp, &compiler))return PX_FALSE;
	if (!PX_CompilerAddSource(&compiler, script))
	{
		printf("compiler error:script too long.\n");
		return PX_FALSE;
	}
	if (!PX_CompilerCompile(&compiler, &payload, "piano"))
	{
		printf("compiler error.check your script and try again\n");
		return PX_FALSE;
	}
	printf("compile done\n");
	if (!PX_VMInitialize(&vmInstance, &mp, payload.buffer, payload.usedsize))
	{
		return PX_FALSE;
	}
	PX_VMRegistryHostFunction(&vmInstance, "PianoKeyDown", PX_VM_PianoKeyDown, PX_NULL);
	PX_VMRegistryHostFunction(&vmInstance, "PianoKeyDownIndex", PX_VM_PianoKeyDownIndex, PX_NULL);
	PX_VMRegistryHostFunction(&vmInstance, "Wait", PX_VM_Wait, PX_NULL);
	PX_VMRegistryHostFunction(&vmInstance, "StartTrack", PX_VM_CreateThread, PX_NULL);

	printf("piano model building...\n");
	PX_PianoModelInitialize(&pianoModel, PX_NULL, PX_NULL);
	printf("build succeeded...\n");
	if (!PX_SoundPlayInitialize(&mp, &soundPlay))
	{
		return PX_FALSE;
	}
	if (!PX_AudioInitialize(&soundPlay))
	{
		printf("sound device error.\n");
		return PX_FALSE;
	}
	PX_AudioInitializeHwnd(GetConsoleWindow());
	PX_SoundPlaySetUserRead(&soundPlay, PX_PianoRead, PX_NULL);

	printf("running...\n");
	lastUpdateTime = PX_TimeGetTime();

	PX_VMBeginThreadFunction(&vmInstance, 0, "main", 0, 0);
	while (PX_TRUE)
	{
		px_dword elapsed;
		now = PX_TimeGetTime();
		elapsed = now - lastUpdateTime;
		lastUpdateTime = now;
		PX_VMRunTime(&vmInstance, -1, elapsed);
		if (!PX_VMIsRuning(&vmInstance))
		{
			break;
		}
	}

	
	
}